@@ -244,21 +244,39 @@ def damos_quotas_cons_arg(cmd_args):
 
     return [time_ms, sz_bytes, reset_interval_ms, weights]
 
+def damos_options_to_quota_goal(garg):
+    # garg is the user inputs
+    # garg should be <metric> <target value> [<current value>|<nid>]
+    # [current value] is given for only 'user_input' <metric>
+    # [nid] is given for only node_mem_{used,free}_bp
+    if not len(garg) in [2, 3]:
+        return None, 'Wrong --damos_quota_goal (%s)' % garg
+    metric, target_value, optionals = garg[0], garg[1], garg[2:]
+    current_value = 0
+    nid = None
+    if _damon.DamosQuotaGoal.metric_require_nid(metric):
+        if len(optionals) != 1:
+            return None, 'nid is not given or something else is given'
+        nid = optionals[0]
+    elif metric == 'user_input':
+        if len(optionals) != 1:
+            return None, 'current value is not given or something else is given'
+        current_value = optionals[0]
+    try:
+        return _damon.DamosQuotaGoal(
+                metric=metric, target_value=target_value,
+                current_value=current_value, nid=nid), None
+    except Exception as e:
+        return None, 'DamosQuotaGoal creation fail (%s, %s)' % (garg, e)
+
 def damos_options_to_quotas(quotas, goals):
     gargs = goals
-    # garg should be <metric> <target value> [current value]
-    # [current value] is given for only 'user_input' <metric>
+    goals = []
     for garg in gargs:
-        if not len(garg) in [2, 3]:
-            return None, 'Wrong --damos_quota_goal (%s)' % garg
-        if garg[0] == 'user_input' and len(garg) != 3:
-            return None, 'Wrong --damos_quota_goal (%s)' % garg
-        if garg[0] != 'user_input' and len(garg) != 2:
-            return None, 'Wrong --damos_quota_goal (%s)' % garg
-    try:
-        goals = [_damon.DamosQuotaGoal(*garg) for garg in gargs]
-    except Exception as e:
-        return None, 'Wrong --damos_quota_goal (%s, %s)' % (gargs, e)
+        goal, err = damos_options_to_quota_goal(garg)
+        if err is not None:
+            return None, 'quota goals parsing fail (%s)' % err
+        goals.append(goal)
 
     qargs = quotas
     if len(qargs) > 6:
@@ -749,7 +767,7 @@ def set_damos_argparser(parser, hide_help):
             default=[],
             metavar='<metric or value>',
             help=' '.join([
-                'damos quota goal (<metric> <target value> [current value]).',
+                'damos quota goal (<metric> <target value> [optional value]).',
                 '<metric> should be {%s}.' %
                 ','.join(_damon.qgoal_metrics)])
             if not hide_help else argparse.SUPPRESS)
